@@ -14,7 +14,6 @@
 import json
 
 from django.utils.translation import ugettext_lazy as _
-import six
 
 from horizon import exceptions
 from horizon import forms
@@ -27,6 +26,7 @@ import sahara_dashboard.content.data_processing. \
     clusters.clusters.workflows.create as c_flow
 from sahara_dashboard.content.data_processing.utils \
     import acl as acl_utils
+from sahara_dashboard.content.data_processing.utils import helpers
 import sahara_dashboard.content.data_processing. \
     utils.workflow_helpers as whelpers
 
@@ -55,15 +55,15 @@ class JobExecutionGeneralConfigAction(workflows.Action):
         super(JobExecutionGeneralConfigAction, self).__init__(request,
                                                               *args,
                                                               **kwargs)
-
-        if request.REQUEST.get("job_id", None) is None:
+        req = request.GET or request.POST
+        if req.get("job_id", None) is None:
             self.fields["job"] = forms.ChoiceField(
                 label=_("Job"))
             self.fields["job"].choices = self.populate_job_choices(request)
         else:
             self.fields["job"] = forms.CharField(
                 widget=forms.HiddenInput(),
-                initial=request.REQUEST.get("job_id", None))
+                initial=req.get("job_id", None))
 
     def populate_job_input_choices(self, request, context):
         return self.get_data_source_choices(request, context)
@@ -119,10 +119,11 @@ class JobExecutionExistingGeneralConfigAction(JobExecutionGeneralConfigAction):
             clusters = []
             exceptions.handle(request,
                               _("Unable to fetch clusters."))
-
-        choices = [(cluster.id, cluster.name)
-                   for cluster in clusters]
-
+        choices = [(cl.id, "%s %s" % (cl.name,
+                                      helpers.ALLOWED_STATUSES.get(cl.status)))
+                   for cl in clusters if cl.status in helpers.ALLOWED_STATUSES]
+        if not choices:
+            choices = [(None, _("No clusters available"))]
         return choices
 
     class Meta(object):
@@ -159,6 +160,7 @@ def _merge_interface_with_configs(interface, job_configs):
 
 class JobConfigAction(workflows.Action):
     MAIN_CLASS = "edp.java.main_class"
+    STORM_PYLEUS_TOPOLOGY_NAME = "topology_name"
     JAVA_OPTS = "edp.java.java_opts"
     EDP_MAPPER = "edp.streaming.mapper"
     EDP_REDUCER = "edp.streaming.reducer"
@@ -192,12 +194,20 @@ class JobConfigAction(workflows.Action):
     main_class = forms.CharField(label=_("Main Class"),
                                  required=False)
 
+    topology_name = forms.CharField(
+        label=_("Topology Name"),
+        help_text=_("Use the same topology name as defined in your "
+                    ".yaml file"),
+        required=False)
+
     java_opts = forms.CharField(label=_("Java Opts"),
                                 required=False)
 
-    streaming_mapper = forms.CharField(label=_("Mapper"))
+    streaming_mapper = forms.CharField(label=_("Mapper"),
+                                       required=False)
 
-    streaming_reducer = forms.CharField(label=_("Reducer"))
+    streaming_reducer = forms.CharField(label=_("Reducer"),
+                                        required=False)
 
     hbase_common_lib = forms.BooleanField(
         label=_("Use HBase Common library"),
@@ -225,10 +235,15 @@ class JobConfigAction(workflows.Action):
 
     def __init__(self, request, *args, **kwargs):
         super(JobConfigAction, self).__init__(request, *args, **kwargs)
-        job_ex_id = request.REQUEST.get("job_execution_id")
+        req = request.GET or request.POST
+        job_ex_id = req.get("job_execution_id")
         if job_ex_id is not None:
             job_ex = saharaclient.job_execution_get(request, job_ex_id)
-            job = saharaclient.job_get(request, job_ex.job_id)
+            try:
+                jt_id = job_ex.job_template_id  # typical APIv2
+            except AttributeError:
+                jt_id = job_ex.job_id  # APIv1.1, older APIv2
+            job = saharaclient.job_get(request, jt_id)
             job_configs, interface_args = _merge_interface_with_configs(
                 job.interface, job_ex.job_configs)
             edp_configs = {}
@@ -250,6 +265,9 @@ class JobConfigAction(workflows.Action):
             if self.MAIN_CLASS in edp_configs:
                 self.fields['main_class'].initial = (
                     edp_configs[self.MAIN_CLASS])
+            if self.STORM_PYLEUS_TOPOLOGY_NAME in edp_configs:
+                self.fields['topology_name'].initial = (
+                    edp_configs[self.STORM_PYLEUS_TOPOLOGY_NAME])
             if self.JAVA_OPTS in edp_configs:
                 self.fields['java_opts'].initial = (
                     edp_configs[self.JAVA_OPTS])
@@ -289,7 +307,8 @@ class JobConfigAction(workflows.Action):
         return cleaned_data
 
     def populate_property_name_choices(self, request, context):
-        job_id = request.REQUEST.get("job_id") or request.REQUEST.get("job")
+        req = request.GET or request.POST
+        job_id = req.get("job_id") or req.get("job")
         job_type = saharaclient.job_get(request, job_id).type
         job_configs = (
             saharaclient.job_get_configs(request, job_type).job_config)
@@ -299,7 +318,7 @@ class JobConfigAction(workflows.Action):
 
     def clean_edp_configs(self, configs):
         edp_configs = {}
-        for key, value in six.iteritems(configs):
+        for key, value in configs.items():
             if key.startswith(self.EDP_PREFIX):
                 edp_configs[key] = value
         for rmkey in edp_configs.keys():
@@ -309,6 +328,7 @@ class JobConfigAction(workflows.Action):
                          self.EDP_MAPPER,
                          self.EDP_REDUCER,
                          self.MAIN_CLASS,
+                         self.STORM_PYLEUS_TOPOLOGY_NAME,
                          self.JAVA_OPTS,
                          self.EDP_ADAPT_FOR_OOZIE,
                          self.EDP_ADAPT_SPARK_SWIFT,
@@ -388,6 +408,10 @@ class JobConfig(workflows.Step):
                 context["job_config"]["configs"][
                     JobConfigAction.EDP_ADAPT_SPARK_SWIFT] = (
                     data.get("adapt_spark_swift", True))
+        elif job_type == "Storm.Pyleus":
+            context["job_config"]["configs"][
+                JobConfigAction.STORM_PYLEUS_TOPOLOGY_NAME] = (
+                data.get("topology_name", ""))
         elif job_type == "MapReduce.Streaming":
             context["job_config"]["configs"][JobConfigAction.EDP_MAPPER] = (
                 data.get("streaming_mapper", ""))
@@ -442,11 +466,15 @@ class JobExecutionInterfaceConfigAction(workflows.Action):
         job = saharaclient.job_get(request, job_id)
         interface = job.interface or []
         interface_args = {}
-
-        job_ex_id = request.REQUEST.get("job_execution_id")
+        req = request.GET or request.POST
+        job_ex_id = req.get("job_execution_id")
         if job_ex_id is not None:
             job_ex = saharaclient.job_execution_get(request, job_ex_id)
-            job = saharaclient.job_get(request, job_ex.job_id)
+            try:
+                jt_id = job_ex.job_template_id  # typical APIv2
+            except AttributeError:
+                jt_id = job_ex.job_id  # APIv1.1, older APIv2
+            job = saharaclient.job_get(request, jt_id)
             job_configs, interface_args = _merge_interface_with_configs(
                 job.interface, job_ex.job_configs)
 
@@ -512,11 +540,11 @@ class LaunchJob(workflows.Workflow):
         return True
 
 
-class SelectHadoopPluginAction(t_flows.SelectPluginAction):
+class SelectPluginForJobLaunchAction(t_flows.SelectPluginAction):
     def __init__(self, request, *args, **kwargs):
-        super(SelectHadoopPluginAction, self).__init__(request,
-                                                       *args,
-                                                       **kwargs)
+        super(SelectPluginForJobLaunchAction, self).__init__(request,
+                                                             *args,
+                                                             **kwargs)
         self.fields["job_id"] = forms.ChoiceField(
             label=_("Plugin name"),
             initial=request.GET.get("job_id") or request.POST.get("job_id"),
@@ -533,8 +561,8 @@ class SelectHadoopPluginAction(t_flows.SelectPluginAction):
         self.fields["job_params"] = forms.ChoiceField(
             label=_("Job params"),
             widget=forms.HiddenInput(attrs={"class": "hidden_create_field"}))
-
-        job_ex_id = request.REQUEST.get("job_execution_id")
+        req = request.GET or request.POST
+        job_ex_id = req.get("job_execution_id")
         if job_ex_id is not None:
             self.fields["job_execution_id"] = forms.ChoiceField(
                 label=_("Job Execution ID"),
@@ -562,7 +590,7 @@ class SelectHadoopPluginAction(t_flows.SelectPluginAction):
 
 
 class SelectHadoopPlugin(workflows.Step):
-    action_class = SelectHadoopPluginAction
+    action_class = SelectPluginForJobLaunchAction
 
 
 class ChosePluginVersion(workflows.Workflow):
